pub enum Error {
Show 29 variants
AnyhowError(Error),
BalanceParsing(String),
CallContractError(String),
ContractMetadata(String),
CommonError(Error),
DryRunUploadContractError(String),
DryRunCallContractError(String),
HexParsing(String),
HttpError(Error),
IncorrectArguments {
expected: usize,
provided: usize,
},
InstallContractsNode(String),
InstantiateContractError(String),
InvalidConstructorName(String),
InvalidMessageName(String),
InvalidName(String),
InvalidToolchain(String),
IO(Error),
ManifestPath(String),
MapAccountError(String),
MissingArgument(String),
NewContract(String),
ParseError(ParseError),
RepositoryMissing,
SerdeJson(Error),
SourcingError(Error),
TestCommand(String),
UnsupportedPlatform {
os: &'static str,
},
UploadContractError(String),
Verification(String),
}Expand description
Represents the various errors that can occur in the crate.
Variants
AnyhowError(Error)
An error occurred.
BalanceParsing(String)
Failed to parse a balance value from a string representation.
CallContractError(String)
Failed to call the smart contract.
ContractMetadata(String)
A contract metadata error happened
CommonError(Error)
A common error originating from pop_common.
DryRunUploadContractError(String)
Dry-run contract upload failed.
DryRunCallContractError(String)
Dry-run contract call failed.
HexParsing(String)
Failed to parse hex-encoded bytes.
HttpError(Error)
An HTTP request failed.
IncorrectArguments
The number of provided arguments did not match the expected count.
InstallContractsNode(String)
Failed to install the contracts node binary.
InstantiateContractError(String)
Contract instantiation failed.
InvalidConstructorName(String)
The specified constructor name is invalid.
InvalidMessageName(String)
The specified message name is invalid.
InvalidName(String)
The specified name is invalid.
InvalidToolchain(String)
The current toolchain is invalid
IO(Error)
An I/O error occurred.
ManifestPath(String)
An I/O error occurred.
MapAccountError(String)
Error returned when mapping an account fails.
MissingArgument(String)
A required argument was not provided.
NewContract(String)
An error occurred while creating a new contract project.
ParseError(ParseError)
An error occurred while parsing a URL.
RepositoryMissing
The Repository property is missing from the template variant.
SerdeJson(Error)
A [serde_json::Error] error occurred
SourcingError(Error)
An error occurred sourcing a binary.
TestCommand(String)
An error occurred while executing a test command.
UnsupportedPlatform
The platform is unsupported.
UploadContractError(String)
An error occurred while uploading the contract.
Verification(String)
An error occurred while verifying a contract