pub struct BuildInfo {
pub rust_toolchain: String,
pub cargo_contract_version: Version,
pub build_mode: BuildMode,
}Expand description
Information about the settings used to build a particular ink! contract.
Note that this should be an optional part of the metadata since it may not necessarily translate to other languages (e.g. ask!, Solidity).
Fields
rust_toolchain: StringThe Rust toolchain used to build the contract.
cargo_contract_version: VersionThe version of cargo-contract used to build the contract.
build_mode: BuildModeThe type of build that was used when building the contract.