pub struct ContractFunction {
pub label: String,
pub payable: bool,
pub args: Vec<Param>,
pub docs: String,
pub default: bool,
pub mutates: bool,
}Expand description
Describes a contract function.
Fields
label: StringThe label of the function.
payable: boolIf the function accepts any value from the caller.
args: Vec<Param>The parameters of the deployment handler.
docs: StringThe function documentation.
default: boolIf the message/constructor is the default for off-chain consumers (e.g UIs).
mutates: boolIf the message is allowed to mutate the contract state. true for constructors.