pub struct ContractMetadata {
pub source: Source,
pub contract: Contract,
pub image: Option<String>,
pub user: Option<User>,
pub abi: Map<String, Value>,
}Expand description
Smart contract metadata.
Fields
source: SourceInformation about the contract’s binary.
contract: ContractMetadata about the contract.
image: Option<String>If the contract is meant to be verifiable, then the Docker image is specified.
user: Option<User>Additional user-defined metadata.
abi: Map<String, Value>Raw JSON of the contract’s abi metadata, generated during contract compilation.
Implementations
Sourceimpl ContractMetadata
impl ContractMetadata
Sourcepub fn new(
source: Source,
contract: Contract,
image: Option<String>,
user: Option<User>,
abi: Map<String, Value>,
) -> ContractMetadata
pub fn new( source: Source, contract: Contract, image: Option<String>, user: Option<User>, abi: Map<String, Value>, ) -> ContractMetadata
Construct new contract metadata.
pub fn remove_source_contract_binary_attribute(&mut self)
Sourcepub fn load<P>(metadata_path: P) -> Result<ContractMetadata, Error>
pub fn load<P>(metadata_path: P) -> Result<ContractMetadata, Error>
Reads the file and tries to parse it as instance of ContractMetadata.
Sourcepub fn check_ink_compatibility(&self) -> Result<(), Error>
pub fn check_ink_compatibility(&self) -> Result<(), Error>
Checks whether the contract’s ink! version is compatible with the
cargo-contract binary.